#include <assert.h>
#include "Patient.h"
#include "Doctor.h"
#include "Hospital.h"
#include "Measurement.h"

Patient::Patient() : mAssignedDoctor(NULL)
{}

Patient::~Patient()
{
    // Release from hospital if in hospital:
    releaseFromHospital();

    // Destroy owned measurements:
    for(Measurements::iterator it=mMeasurements.begin();
        it != mMeasurements.end();
        ++it)
        {
            Measurement *m = *(it);
            delete m;
        }
}

void Patient::releaseFromHospital() 
{
    if (mAssignedDoctor)
        mAssignedDoctor->dropPatient(this);
    Hospital::theHospital().removePatient(this);
}

void Patient::printMeasurements(std::ostream &os) const
{
    for(Measurements::const_iterator it = mMeasurements.begin();
        it != mMeasurements.end();
        ++it)
        {
            os << "------------------------------" << endl;
            os << (**it) << endl;
        }
}


std::ostream operator<<(std::ostream &os,Patient const &p)
{
    os << "Patient: " << p.getName() << " / " << p.getAccountNo() << endl;
    os << "Assigned doctor: ";
    if (p.getDoctor()) 
        os << p.getDoctor()->getName() << endl;
    else
        os << "(none)" << endl;
    os << "History: " << endl;
    p.printMeasurements(os);
    return os;
}
